/*
// 24 dec 2003
*/
#include <stdio.h>      // perror
#include <stdlib.h>     // EXIT_FAILURE
#include <sys/types.h>  // pid_t
#include <netinet/in.h> // sockaddr_in
#include <stdarg.h> // va_list
#include <string.h> // strdup
#include <unistd.h> // pipe, execlp
#include <signal.h> // SIGKILL
#include <sys/wait.h> // wait
#include <time.h>     // time
#include <sys/socket.h>   // socket

/*
// Avoid duplicate calls
*/
#ifndef SOCKSERVER_H
#include "sockserver.h"
#define SOCKSERVER_H
#endif

char * make_message
(
  const char *fmt,
  ...
)
{
  int     n;
  int     size;
  char *  p;
  va_list ap;

  p = (char *) strdup("");

  size = 100;
  if ((p = (char *) malloc(size)) == NULL)
  {
    return NULL;
  }
  while (1)
  {
    va_start(ap, fmt);
    n = vsnprintf(p, size, fmt, ap);
    va_end(ap);

    if (n > -1)
    {
/*
// Worked -> return string
*/
      if (n < size)
      {
        return p;
      }
/*
// glibc 2.1 (precise size reallocation)
*/
      if (n >= size)
      {
        size = n + 1;
      }
    }

/*
// glibc 2.0 (double size reallocation)
*/
    if (n <= -1)
    {
      size *= 2;
    }

/*
// Reallocate string space
*/
    if ((p = (char *) realloc(p, size)) == NULL)
    {
      return NULL;
    }
  }

  return p;
}

int getscanflag
(
  char * pathname
)
{
  int flag;
  int pid;   // process to call scanner
  int fd[2]; // pipe
  int status;
  char * response;
  char * output;
  char * arg;
/*
  char * pathvar;
  char * newpath[256];
*/
  arg = strdup(pathname);
  return system(make_message("%s \"%s\"", SCANNERWRAPPER, pathname));

/*
// Initialisation
*/
  response = (char *) calloc(1, sizeof(char));
  output   = strdup("");

  flag = 0;
  if (pipe(fd) != 0)
  {
    perror("");
    return flag;
  }

  pid = fork();
/*
// Child
*/
  if (pid == 0)
  {
    close(fd[1]);
    close(0);
    dup(fd[0]);

    for (;;)
    {
      if ((status = read(fd[0], response, sizeof(char))) == 0)
      {
        break;
      }
      output = make_message("%s%c", output, response[0]);
    }
    if (strcmp(output, "1") == 0)
    {
      flag = 1;
    }

    close(fd[0]);

    return flag;
  }
/*
// Parent
*/
  if (pid > 0)
  {
    close(fd[0]);
    close(1);
    dup(fd[1]);
    close(fd[1]);
/*
    pathvar = getenv("PATH");
    strcpy(newpath, pathvar);
    strcat(newpath, ":/");
    setenv("PATH", newpath, 1);
*/
    if (execlp(SCANNERWRAPPER, arg, NULL) == -1)
    {
      return flag;
    }

    wait(&status);
  }

  return flag;
}

void serve_connection
(
  int socket_value
)
{
  char         character;
  char *       input;
  int          counter;
  int          status;

  counter = 0;
  input = calloc(1, sizeof(char));
  input[counter] = '\0';

  for (;;)
  {
    status = read(socket_value, &(character), sizeof(char));
    if (status == 0)
    {
      break;
    }
    if ((character == '\0') ||
        (character == 0) ||
        (character == '\n'))
    {
      status = getscanflag(input);
      if (status != 0)
      {
        write(socket_value, "1", strlen("1"));
      }
      if (status == 0)
      {
        write(socket_value, "0", strlen("0"));
      }
      free(input);
      break;
    }
    input = make_message("%s%c", input, character);
  }

  return;
}

void settimer
(
)
{
  time_t t1;
  time_t t2;

  (void) time(&t1);
  while (1)
  {
    (void) time(&t2);
    if ((((int) t2 - t1) < 0) ||
        (((int) t2 - t1) >= TIMELIMIT))
    {
      kill(0, SIGKILL);
      break;
    }
  }
  return;
}

void netinit
(
)
{
  struct sockaddr_in server_address;
  int                server_socket;
  struct sockaddr_in client_address;
  int                client_address_size;
  int                current_socket;
  pid_t              pid;
  int                status;

  bzero((char *) &(server_address), sizeof(server_address));
  server_address.sin_family      = AF_INET;
  server_address.sin_port        = htons(SERVER_TCP_PORT);
  server_address.sin_addr.s_addr = htonl(SERVER_IP_ADDRESS);

  if ((server_socket = socket(PF_INET, SOCK_STREAM, 0)) < 0)
  {
    perror("socket");
    exit(EXIT_FAILURE);
  }

  if (bind
      (
        server_socket,
        (struct sockaddr *) (&(server_address)),
        sizeof(server_address)
      ) < 0)
  {
    perror("bind");
    exit(EXIT_FAILURE);
  }

  listen(server_socket, 5);

  for (;;)
  {
    client_address_size = sizeof(client_address);
    current_socket =
      accept
      (
        server_socket,
        (struct sockaddr *) &(client_address),
        &(client_address_size)
      );

    if (current_socket < 0)
    {
      perror("accept");
      exit(EXIT_FAILURE);
    }

/*
// Making respawning multiple processes
*/
    pid = fork();
    if (pid == 0)
    {
      pid = fork();
      if (pid == 0)
      {
        close(server_socket);
        serve_connection(current_socket);
        exit(EXIT_SUCCESS);
      }
      if (pid > 0)
      {
/*
// Self destruct process if time runs out
//        settimer();
*/
      }
      exit(EXIT_SUCCESS);
    }

    close(current_socket);
    wait(&status);
  }

  return;
}

int main
(
)
{
  pid_t pid;

  pid = fork();
/*
// Child
*/
  if (pid == 0)
  {
    netinit();
  }

  return 0;
}
